﻿Imports System.ComponentModel
Imports System.ComponentModel.DataAnnotations

Public Class Car
    Implements INotifyPropertyChanged
    Implements IDataErrorInfo

    Private _marke As String
    Public Property Marke() As String
        Get
            Return _marke
        End Get
        Set(ByVal value As String)
            _marke = value
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Marke"))
        End Set
    End Property

    Private _modell As String
    Public Property Modell() As String
        Get
            Return _modell
        End Get
        Set(ByVal value As String)
            _modell = value
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Modell"))
        End Set
    End Property

    Private _price As Decimal
    Public Property Price() As Decimal
        Get
            Return _price
        End Get
        Set(ByVal value As Decimal)
            _price = value
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Price"))
        End Set
    End Property

    Default Public ReadOnly Property Item(columnName As String) As String Implements IDataErrorInfo.Item
        Get
            Dim valRes As ValidationResult = ValidationErrors.Where(Function(v) v.MemberNames.Contains(columnName) = True).FirstOrDefault
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(NameOf(ErrorsList)))

            If valRes Is Nothing Then Return Nothing

            Return valRes.ErrorMessage
        End Get
    End Property

    Public Function ValidationErrors() As List(Of ValidationResult)
        Dim valRet As New List(Of ValidationResult)

        If String.IsNullOrEmpty(Marke) OrElse String.IsNullOrEmpty(Modell) Then
            Dim fieldList As New List(Of String)
            If String.IsNullOrEmpty(Marke) Then fieldList.Add(NameOf(Marke))
            If String.IsNullOrEmpty(Modell) Then fieldList.Add(NameOf(Modell))
            valRet.Add(New ValidationResult("Die Pflichtfelder müssen alle ausgefüllt werden!", fieldList))
        Else
            If Marke.Length < 3 Then valRet.Add(New ValidationResult($"'{NameOf(Marke)}' muss aus mindestens 2 Zeichen bestehen!", New List(Of String) From {NameOf(Marke)}))
            If Modell.Length < 3 Then valRet.Add(New ValidationResult($"'{NameOf(Modell)}' muss aus mindestens 3 Zeichen bestehen!", New List(Of String) From {NameOf(Modell)}))
        End If

        If Price < 0 Then valRet.Add(New ValidationResult($"'{NameOf(Price)}' darf nicht kleiner als 0 sein!", New List(Of String) From {NameOf(Price)}))
        Return valRet
    End Function

    Public ReadOnly Property ErrorsList As List(Of String)
        Get
            Return ValidationErrors.Select(Function(e) e.ErrorMessage).ToList()
        End Get
    End Property


    Public ReadOnly Property [Error] As String Implements IDataErrorInfo.Error
        Get
            Return Nothing
        End Get
    End Property

    Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
End Class
